from unittest import TestCase

from formencode import Invalid
from pyramid.httpexceptions import HTTPBadRequest
from pyramid.testing import DummyRequest

from .. import (Form,
                FormNotValidated, FormInvalid,
                csrf_field)

from .utils import (DummySchema, LooseDummySchema, NestedDummySchema,
                    DummyObject, dummy_csrf_token)


class TestForm(TestCase):

    def test_validate_success(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema())
        self.assertTrue(form.validate(skip_csrf=True))

    def test_validate_failure(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

        self.assertIn('foo', form.errors)

    def test_validate_get(self):
        request = DummyRequest(params={'foo': 'hello'})
        form = Form(request, DummySchema(), method='GET')
        self.assertTrue(form.validate(skip_csrf=True))

    def test_validate_twice(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))
        self.assertFalse(form.validate(skip_csrf=True))

        self.assertIn('foo', form.errors)

    def test_invalid_non_error_dict(self):
        request = DummyRequest(post={'what': ''})
        form = Form(request, DummySchema())
        with self.assertRaises(Invalid):
            form.validate(skip_csrf=True)

    def test_validate_csrf(self):
        request = DummyRequest(post={
            'foo': 'hello',
            csrf_field: dummy_csrf_token
        })
        form = Form(request, DummySchema())
        self.assertTrue(form.validate())

    def test_validate_method_get_not_allowed(self):
        request = DummyRequest(params={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

    def test_assert_valid(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        with self.assertRaises(AssertionError) as cm:
            form.assert_valid(skip_csrf=True)
        exc = cm.exception
        self.assertIn('foo', str(exc))

    def test_bind(self):
        request = DummyRequest(post={'foo': 'quux'})
        form = Form(request, DummySchema())
        self.assertTrue(form.validate(skip_csrf=True))

        obj = DummyObject()
        form.bind(obj)
        self.assertEqual(obj.foo, 'quux')

    def test_bind_without_validating(self):
        request = DummyRequest(post={'foo': 'quux'})
        form = Form(request, DummySchema())

        obj = DummyObject()
        with self.assertRaises(FormNotValidated):
            form.bind(obj)

    def test_bind_invalid(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

        obj = DummyObject()
        with self.assertRaises(FormInvalid):
            form.bind(obj)

    def test_bind_ignores_private_attributes(self):
        request = DummyRequest(post={
            'foo': 'frobozz',
            '_something': 'blah',
        })
        form = Form(request, LooseDummySchema())
        self.assertTrue(form.validate(skip_csrf=True))

        obj = DummyObject()
        form.bind(obj)
        self.assertEqual(obj.foo, 'frobozz')
        self.assertFalse(hasattr(obj, '_something'))

    def test_bind_nested(self):
        request = DummyRequest(post={
            'name': 'ben bitdiddle',
            'qty': '10',
            'items-0.foo': 'blah blah blah',
            'subfields.foo': 'hello',
        })
        form = Form(request, NestedDummySchema())
        self.assertTrue(form.validate(skip_csrf=True))

        obj = DummyObject()
        obj.items = [DummyObject(), DummyObject()]
        obj.subfields = DummyObject()

        form.bind(obj)
        self.assertEqual(obj.items[0].foo, 'blah blah blah')
        self.assertEqual(obj.subfields.foo, 'hello')
        self.assertEqual(obj.qty, 10)
        self.assertEqual(obj.name, 'ben bitdiddle')

    def test_access_data_without_validating(self):
        request = DummyRequest(post={'foo': 'quux'})
        form = Form(request, DummySchema())

        with self.assertRaises(FormNotValidated):
            form.data

    def test_errors_for(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

        errors = form.errors_for('foo')
        self.assertEqual(len(errors), 1)
        self.assertIn('enter a value', errors[0])

    def test_is_error(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

        self.assertTrue(form.is_error('foo'))

    def test_method_allowed_implicit_method(self):
        request = DummyRequest(method='GET')
        form = Form(request, DummySchema(), method=None)
        self.assertTrue(form.method_allowed)

        request = DummyRequest(method='POST')
        form = Form(request, DummySchema(), method=None)
        self.assertTrue(form.method_allowed)

    def test_method_allowed_explicit_method(self):
        request = DummyRequest(method='GET')
        form = Form(request, DummySchema(), method='POST')
        self.assertFalse(form.method_allowed)

        request = DummyRequest(method='POST')
        form = Form(request, DummySchema(), method='POST')
        self.assertTrue(form.method_allowed)

    def test_validate_csrf_missing(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        with self.assertRaises(HTTPBadRequest):
            form.validate_csrf()

    def test_validate_csrf_wrong(self):
        request = DummyRequest(post={'foo': '', csrf_field: 'wrong'})
        form = Form(request, DummySchema())
        with self.assertRaises(HTTPBadRequest):
            form.validate_csrf()



import logging

import six

from formencode import Invalid

from webhelpers2.html import tags
from webhelpers2.misc import NotGiven
from webhelpers2.html.builder import HTML
from pyramid.httpexceptions import HTTPBadRequest


log = logging.getLogger('pyramid_uniform.validate')

csrf_field = "_authentication_token"


class FormError(Exception):
    """
    Superclass for form-related errors.
    """
    def __init__(self, *args, **kw):
        args = args or (self.message,)
        Exception.__init__(self, *args, **kw)


class FormNotValidated(FormError):
    """
    Raised when form data is used before form has been validated: for example,
    when :py:meth:`.Form.bind` is called.
    """
    message = 'Form has not been validated; call validate() first'


class FormInvalid(FormError):
    """
    Raised when form data is used but the form is not valid.
    """
    message = 'Form is invalid'


class State(object):
    """
    A relatively simple state object for the schema being validated to use,
    with a reference to the request being validated.
    """
    def __init__(self, request):
        self.request = request


class Form(object):
    """
    Represents a set of fields (GET or POST parameters) to be validated using a
    :py:mod:`FormEncode` schema.

    :param request: The web request containing data to be validated
    :type request: :py:class:`WebOb.Request`

    :param schema: The schema to validate against
    :type schema: :py:class:`FormEncode.Schema`

    :param method: HTTP request method that this form expects: GET or POST
    :type method: str
    """
    def __init__(self, request, schema, method='POST', skip_csrf=False):
        self.request = request
        self.schema = schema
        self.method = method
        self.skip_csrf = skip_csrf
        self.dict_char = '.'
        self.list_char = '-'
        self.multipart = True
        self.is_validated = False
        self.errors = {}
        self.state = State(request)

    @property
    def data(self):
        """
        Once the form has been validated, contains the results of that
        validation as a dict.

        :raises FormNotValidated: if the form has not yet been validated
        """
        if self.is_validated:
            return self._data
        raise FormNotValidated

    @property
    def method_allowed(self):
        """
        Is the method that was used to submit this form allowed?

        If this form doesn't have a request method set (i.e., if it was
        explicitly set to ``None``), any method is valid. Otherwise, the
        method of the form submission must match the method required by this
        form.
        """
        if self.method:
            return self.request.method == self.method
        return True

    def validate(self, skip_csrf=False, assert_valid=False):
        """
        Validate a form submission.

        When ``assert_valid`` is ``False`` (the default), a bool will be
        returned to indicate whether the form was valid. (Note: this isn't
        strictly true--a missing or bad CSRF token will result in a
        immediate ``400 Bad Request`` response).

        When ``assert_valid`` is ``True``, certain conditions will be
        asserted. When an assertion fails, an ``AssertionError`` will be
        raised.

        :param skip_csrf: if True, bypass the CSRF check
        :type skip_csrf: bool

        :param assert_valid:
            if True, assert validity instead of returning status
        :type assert_valid: bool
        """
        request = self.request
        if self.is_validated:
            return not self.errors

        if assert_valid:
            assert self.method_allowed, (
                'Expected request method {}; got {} instead'
                .format(self.method, self.request.method))
        elif not self.method_allowed:
            return False

        params = self._get_params()

        if not (self.skip_csrf or skip_csrf):
            self.validate_csrf(params)
            # Remove CSRF token from params; use a copy so the original
            # request params aren't changed.
            params = params.copy()
            params.pop(csrf_field)

        self._data = params

        try:
            self._data = self.schema.to_python(params, self.state)
        except Invalid as e:
            if e.error_dict:
                self.errors = e.unpack_errors(
                    True, self.dict_char, self.list_char)
            else:
                # unpack_errors() will raise an AssertionError when the
                # Invalid exception does not contain an error dict. Since
                # that's not particularly helpful, re-raise the original
                # exception so we at least have some idea of what happened.
                raise e
            log.error("Validation failure on %r from %s [%s]\n"
                      "Params:\n%r\n\nErrors:\n%r\n",
                      request.url,
                      # Work with DummyRequest
                      getattr(request, 'remote_addr', None),
                      getattr(request, 'user_agent', None),
                      params, self.errors)

        self.is_validated = True
        valid = not self.errors

        if assert_valid:
            assert valid, "form invalid with errors: %r" % self.errors

        return valid

    def assert_valid(self, **kw):
        """
        Assert that this form is valid, the request method is appropriate, and
        the CSRF check passes (unless it is explicitly skipped).
        """
        self.validate(assert_valid=True, **kw)

    def validate_csrf(self, params=None):
        """
        Validate that the CSRF token is correct.
        """
        if params is None:
            params = self._get_params()
        if csrf_field not in params:
            raise HTTPBadRequest('Missing CSRF token')
        got_token = params.get(csrf_field)
        wanted_token = self.request.session.get_csrf_token()
        if got_token != wanted_token:
            raise HTTPBadRequest('Bad CSRF token: wanted %r, got %r' %
                                 (wanted_token, got_token))

    def _get_params(self):
        if self.method == 'POST':
            return self.request.POST
        else:
            return self.request.params

    def bind(self, obj):
        """
        Bind the data from this form to an object: that is, try to set
        attributes on the client corresponding to the keys present in the
        validated data. The object can be a template object, SQLAlchemy object,
        etc.

        If any value in the data is a list or another dictionary, recurse with
        that key.

        Private attributes, which is anything that is prefixed with ``_``, will
        be skipped.

        Once done, return the object.
        """
        if not self.is_validated:
            raise FormNotValidated
        if self.errors:
            raise FormInvalid('Form not valid; cannot bind')

        def crud_update(obj, params):
            for k, v in params.items():
                if not k.startswith('_'):
                    if type(v) == list:
                        for ii, el in enumerate(v):
                            crud_update(getattr(obj, k)[ii], el)
                    elif type(v) == dict:
                        crud_update(getattr(obj, k), v)
                    else:
                        setattr(obj, k, v)

        crud_update(obj, self.data)
        return obj

    def errors_for(self, field):
        """
        Return a list of errors for the given field.
        """
        errors = self.errors.get(field, [])
        if isinstance(errors, six.string_types):
            errors = [errors]
        return errors

    def is_error(self, field):
        """
        Check if the given field has any validation errors.
        """
        return field in self.errors


class Renderer(object):
    def __init__(self, data, errors, name_prefix='', id_prefix=''):
        self.data = data
        self.errors = errors
        self.name_prefix = name_prefix
        self.id_prefix = id_prefix

    def text(self, name, value=None, id=NotGiven, **attrs):
        """
        Return a ``text`` input tag.
        """
        name = self._get_name(name)
        return tags.text(name,
                         self.value(name, value),
                         self._get_id(id, name),
                         **attrs)

    def file(self, name, value=None, id=NotGiven, **attrs):
        """
        Return a ``file`` input tag.
        """
        name = self._get_name(name)
        return tags.file(name,
                         self.value(name, value),
                         self._get_id(id, name),
                         **attrs)

    def hidden(self, name, value=None, id=NotGiven, **attrs):
        """
        Return a ``hidden`` input tag.
        """
        name = self._get_name(name)
        if value is None:
            value = self.value(name)

        return tags.hidden(name,
                           value,
                           self._get_id(id, name),
                           **attrs)

    def radio(self, name, value=None, checked=False, label=None, **attrs):
        """
        Return a radio button tag.
        """
        name = self._get_name(name)
        checked = self.value(name) == value or checked
        return tags.radio(name, value, checked, label, **attrs)

    def submit(self, name=None, value=None, id=NotGiven, **attrs):
        """
        Return a submit button tag.
        """
        name = self._get_name(name)
        return tags.submit(name,
                           self.value(name, value),
                           self._get_id(id, name),
                           **attrs)

    def select(self, name, selected_values, options, id=NotGiven, **attrs):
        """
        Return a select tag.
        """
        name = self._get_name(name)
        val = self.value(name, selected_values)
        if not isinstance(val, list):
            val = [val]
        return tags.select(name,
                           val,
                           options,
                           self._get_id(id, name),
                           **attrs)

    def checkbox(self, name, value="1", checked=False, label=None, id=NotGiven,
                 **attrs):
        """
        Return a checkbox tag.
        """
        name = self._get_name(name)
        return tags.checkbox(name,
                             value,
                             self.value(name, checked),
                             label,
                             self._get_id(id, name),
                             **attrs)

    def textarea(self, name, content="", id=NotGiven, **attrs):
        """
        Return a textarea tag.
        """
        name = self._get_name(name)
        return tags.textarea(name,
                             self.value(name, content),
                             self._get_id(id, name),
                             **attrs)

    def password(self, name, value=None, id=NotGiven, **attrs):
        """
        Return a password input tag.
        """
        name = self._get_name(name)
        return tags.password(name, self.value(name, value),
                             self._get_id(id, name),
                             **attrs)

    def is_error(self, name):
        """
        Check if the given field has any validation errors.
        """
        name = self._get_name(name)
        return name in self.errors

    def errors_for(self, name):
        """
        Return a list of errors for the given field.
        """
        name = self._get_name(name)
        return self.form.errors_for(name)

    def errorlist(self, name):
        """
        Return a list of errors for the given field as a ``ul`` tag.
        """
        name = self._get_name(name)
        errors = self.errors_for(name)

        if not errors:
            return ''

        content = "\n".join(HTML.tag("li", error) for error in errors)
        return HTML.tag("ul", tags.literal(content), class_='error')

    def value(self, name, default=None):
        """
        Return the value for the given field as supplied to the form.
        """
        return self.data.get(name, default)

    def _get_id(self, id, name):
        if id and name:
            if id is NotGiven:
                id = self.id_prefix + name
            id = id.replace('.', '_')
            id = tags._make_safe_id_component(id)
            return id

    def _get_name(self, name):
        if name:
            return self.name_prefix + name


class FormRenderer(Renderer):
    """
    Wraps a form to provide HTML rendering capability.
    """
    def __init__(self, form, csrf_field=csrf_field,
                 name_prefix='', id_prefix=''):
        self.form = form
        self.csrf_field = csrf_field

        try:
            data = form.data
        except FormNotValidated:
            data = {}
        Renderer.__init__(self, data=data, errors=form.errors,
                          name_prefix=name_prefix, id_prefix=id_prefix)

    def begin(self, url=None, skip_csrf=False, **attrs):
        """
        Return a ``form`` opening tag.
        """
        url = url or self.form.request.path
        multipart = attrs.pop('multipart', self.form.multipart)
        tag = tags.form(url, multipart=multipart, **attrs)
        if not (self.form.skip_csrf or skip_csrf):
            tag += self.csrf_token()
        return tag

    def end(self):
        """
        Return a ``form`` closing tag.
        """
        return tags.end_form()

    def csrf(self, name=None):
        """
        Return a bare hidden input field containing the CSRF token and param
        name.
        """
        name = name or self.csrf_field

        token = self.form.request.session.get_csrf_token()
        return self.hidden(name, value=token)

    def csrf_token(self, name=None):
        """
        Return a hidden field containing the CSRF token, wrapped in an
        invisible div, so that it is valid HTML regardless of context.
        """
        return HTML.tag('div', self.csrf(name), style='display:none;')



from unittest import TestCase

from pyramid.testing import DummyRequest

from .. import Form, FormRenderer, csrf_field

from .utils import DummySchema, dummy_csrf_token


class TestRenderer(TestCase):

    def test_success_has_empty_errorlist(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema())
        self.assertTrue(form.validate(skip_csrf=True))

        renderer = FormRenderer(form)
        self.assertEqual(renderer.errorlist('foo'), '')

    def test_failure_has_errorlist(self):
        request = DummyRequest(post={'foo': ''})
        form = Form(request, DummySchema())
        self.assertFalse(form.validate(skip_csrf=True))

        renderer = FormRenderer(form)
        self.assertIn('enter a value', renderer.errorlist('foo'))

    def _make_renderer(self, value='hello'):
        request = DummyRequest(post={'foo': value})
        form = Form(request, DummySchema())
        form.validate(skip_csrf=True)
        return FormRenderer(form)

    def test_value(self):
        renderer = self._make_renderer()
        self.assertEqual(renderer.value('foo'), 'hello')

    def test_text_without_param(self):
        renderer = self._make_renderer()
        tag = renderer.text('bar', 'somevalue')
        self.assertIn('<input', tag)
        self.assertIn('bar', tag)
        self.assertIn('somevalue', tag)

    def test_text_with_param(self):
        renderer = self._make_renderer()
        tag = renderer.text('foo', 'somevalue')
        self.assertIn('<input', tag)
        self.assertIn('foo', tag)
        self.assertIn('hello', tag)
        self.assertNotIn('somevalue', tag)

    def test_text_without_id(self):
        renderer = self._make_renderer()
        tag = renderer.text('foo', 'somevalue', id=None)
        self.assertEqual(tag,
                         '<input name="foo" type="text" value="hello" />')

    def test_file(self):
        renderer = self._make_renderer()
        tag = renderer.file('hello')
        self.assertEqual(tag,
                         '<input id="hello" name="hello" type="file" />')

    def test_hidden(self):
        renderer = self._make_renderer()
        tag = renderer.hidden('hello')
        self.assertEqual(tag,
                         '<input id="hello" name="hello" type="hidden" />')

    def test_radio(self):
        renderer = self._make_renderer()
        tag = renderer.radio('hello', 'a')
        self.assertEqual(
            tag,
            '<input id="hello_a" name="hello" type="radio" value="a" />')

    def test_radio_selected(self):
        renderer = self._make_renderer()
        tag = renderer.radio('hello', 'b', checked=True)
        self.assertEqual(
            tag,
            '<input checked="checked" id="hello_b" '
            'name="hello" type="radio" value="b" />')

    def test_submit(self):
        renderer = self._make_renderer()
        tag = renderer.submit('hello')
        self.assertEqual(
            tag,
            '<input id="hello" name="hello" type="submit" />')

    def test_select(self):
        renderer = self._make_renderer()
        tag = renderer.select('hello', None, [('a', 12), ('b', 24)])
        self.assertEqual(
            tag,
            '<select id="hello" name="hello">\n'
            '<option value="a">12</option>\n'
            '<option value="b">24</option>\n'
            '</select>')

    def test_select_scalar_selected(self):
        renderer = self._make_renderer()
        tag = renderer.select('hello', 'a', [('a', 12), ('b', 24)])
        self.assertEqual(
            tag,
            '<select id="hello" name="hello">\n'
            '<option selected="selected" value="a">12</option>\n'
            '<option value="b">24</option>\n'
            '</select>')

    def test_select_multiple(self):
        renderer = self._make_renderer()
        tag = renderer.select('hello', 'a', [('a', 12), ('b', 24)])
        self.assertEqual(
            tag,
            '<select id="hello" name="hello">\n'
            '<option selected="selected" value="a">12</option>\n'
            '<option value="b">24</option>\n'
            '</select>')

    def test_select_list_selected(self):
        renderer = self._make_renderer()
        tag = renderer.select('hello', ['a', 'b'], [('a', 12), ('b', 24)])
        self.assertEqual(
            tag,
            '<select id="hello" name="hello">\n'
            '<option selected="selected" value="a">12</option>\n'
            '<option selected="selected" value="b">24</option>\n'
            '</select>')

    def test_checkbox(self):
        renderer = self._make_renderer()
        tag = renderer.checkbox('hello')
        self.assertEqual(
            tag,
            '<input id="hello" name="hello" type="checkbox" value="1" />')

    def test_checkbox_checked(self):
        renderer = self._make_renderer()
        tag = renderer.checkbox('hello', checked=True)
        self.assertEqual(
            tag,
            '<input checked="checked" id="hello" '
            'name="hello" type="checkbox" value="1" />')

    def test_textarea(self):
        renderer = self._make_renderer()
        tag = renderer.textarea('hello')
        self.assertEqual(
            tag,
            '<textarea id="hello" name="hello"></textarea>')

    def test_password(self):
        renderer = self._make_renderer()
        tag = renderer.password('hello')
        self.assertEqual(
            tag,
            '<input id="hello" name="hello" type="password" />')

    def test_is_error(self):
        renderer = self._make_renderer('')
        self.assertTrue(renderer.is_error('foo'))

        renderer = self._make_renderer('hello')
        self.assertFalse(renderer.is_error('foo'))

    def test_errors_for(self):
        renderer = self._make_renderer('')
        errors = renderer.errors_for('foo')
        self.assertEqual(len(errors), 1)
        self.assertIn('enter a value', errors[0])

        renderer = self._make_renderer('hello')
        self.assertEqual(renderer.errors_for('foo'), [])

    def test_begin(self):
        renderer = self._make_renderer()
        tag = renderer.begin()
        self.assertIn('<form', tag)
        self.assertIn(dummy_csrf_token, tag)

    def test_begin_skip_csrf(self):
        renderer = self._make_renderer()
        tag = renderer.begin(skip_csrf=True)
        self.assertIn('<form', tag)
        self.assertNotIn(dummy_csrf_token, tag)

    def test_form_skip_csrf(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema(), skip_csrf=True)
        form.validate(skip_csrf=True)
        renderer = FormRenderer(form)
        tag = renderer.begin()
        self.assertIn('<form', tag)
        self.assertNotIn(dummy_csrf_token, tag)

    def test_end(self):
        renderer = self._make_renderer()
        tag = renderer.end()
        self.assertEqual(tag, '</form>')

    def test_csrf(self):
        renderer = self._make_renderer()
        tag = renderer.csrf()
        self.assertNotIn('div', tag)
        self.assertIn('<input', tag)
        self.assertIn(dummy_csrf_token, tag)

    def test_csrf_token(self):
        renderer = self._make_renderer()
        tag = renderer.csrf_token()
        self.assertIn('<input', tag)
        self.assertIn('<div', tag)
        self.assertIn(dummy_csrf_token, tag)
        self.assertIn(csrf_field, tag)

    def test_explicit_id(self):
        renderer = self._make_renderer()
        tag = renderer.text('foo', id='tiger')
        self.assertIn('id="tiger"', tag)

    def test_id_prefix(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema())
        form.validate(skip_csrf=True)
        renderer = FormRenderer(form, id_prefix='blorg-')
        tag = renderer.text('foo')
        self.assertIn('id="blorg-foo"', tag)

    def test_renderer_unvalidated_form(self):
        request = DummyRequest()
        form = Form(request, DummySchema())
        renderer = FormRenderer(form)
        tag = renderer.text('hello')
        self.assertEqual(
            tag,
            '<input id="hello" name="hello" type="text" />')

    def test_name_prefix(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema())
        renderer = FormRenderer(form, name_prefix='partial.')
        tag = renderer.text('foo')
        self.assertIn('name="partial.foo"', tag)

    def test_name_prefix_id_generation(self):
        request = DummyRequest(post={'foo': 'hello'})
        form = Form(request, DummySchema())
        renderer = FormRenderer(form, name_prefix='partial.')
        tag = renderer.text('foo')
        self.assertIn('name="partial.foo"', tag)
        self.assertIn('id="partial_foo', tag)

    def test_none_name(self):
        renderer = self._make_renderer()
        tag = renderer.text(None, 'somevalue')
        self.assertIn('<input', tag)
        self.assertNotIn('name="', tag)
        self.assertIn('somevalue', tag)



from formencode import ForEach, Schema, NestedVariables, validators
from pyramid.testing import DummySession


# This always stays the same.
dummy_csrf_token = DummySession().get_csrf_token()


class DummySchema(Schema):
    allow_extra_fields = False
    foo = validators.String(not_empty=True)


class LooseDummySchema(DummySchema):
    allow_extra_fields = True


class DummyObject(object):
    pass


class NestedDummySchema(Schema):
    allow_extra_fields = False
    pre_validators = [NestedVariables]
    items = ForEach(DummySchema)
    subfields = DummySchema
    name = validators.String(not_empty=True)
    qty = validators.Int(min=4, max=100)



from setuptools import setup, find_packages


setup(name='pyramid_uniform',
      version='0.3',
      description='Form handling for Pyramid.',
      long_description='',
      classifiers=[
          'Development Status :: 2 - Pre-Alpha',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Framework :: Pyramid',
      ],
      keywords='pyramid forms validation rendering',
      url='http://github.com/cartlogic/pyramid_uniform',
      author='Scott Torborg',
      author_email='scott@cartlogic.com',
      install_requires=[
          'Pyramid',
          'FormEncode',
          'webhelpers2',
          'six',
          # These are for tests.
          'coverage',
          'nose>=1.1',
          'nose-cover3',
      ],
      license='MIT',
      packages=find_packages(),
      test_suite='nose.collector',
      tests_require=['nose'],
      include_package_data=True,
      zip_safe=False)
